package data;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.util.Vector;


/** Defining the interface for a dimension of the data.
 */
public abstract class DataDimension {

  private String    m_pName;
  

  //statistics
  private float     m_fDimMin, m_fDimMax;         //minimum and maximum values
  private int       m_iDimMinPos, m_iDimMaxPos;   //minimum and maximum positions
  private float     m_fDimMean;             //mean value of the dimension
  private int       m_iEntryNum;

  ////////////////////////////////////
  // GET METHODS                    //
  ////////////////////////////////////

  /** Returns the minimum value of the dimension.
   *
   * @return Minimum value of the specified dimension.
   */
  public float getMinimumValue() {
    return m_fDimMin;
  }


  /** Returns the maximum value of the dimension.
   *
   * @return Maximum value of the specified dimension.
   */
  public float getMaximumValue() {
    return m_fDimMax;
  }

  /** Returns the arithmetic mean of the dimension.
   *
   * @return Arithmetic mean  of the specified dimension.
   */
  public float getMean() {
    return m_fDimMean;
  }

  /** Returns the values of the dimension as float arry.
   *
   * @return float array holding a value for each data item.
   */
  public abstract float[] getValues();

  /** Returns the number of data items (= entries).
   *
   * @return Number of data items.
   */
  public int getEntryNum() {
    return m_iEntryNum;
  }

  /** Returns the name of the dimension.
   *
   * @return Name of the dimension.
   */
  public String getName() {
    return m_pName;
  }

  /** Returns the String representation of the value of the specified data item in this dimension.
   *
   * @return String representation of the value of the specified data item in this dimension.
   */
  public abstract String getStringValue(int iEntryId);

  
  //////////////////////////////////////////
  // PROTECTED SET METHODS FOR SUBCLASSES //
  //////////////////////////////////////////
  protected void setDimensionInfo(
    String pName, 
    int iEntryNum, 
    float fDimMin, 
    float fDimMax, 
    int iDimMinPos, 
    int iDimMaxPos,    
    float fDimMean
  ) {
  
    //just set the members
    m_pName = pName;
    m_iEntryNum = iEntryNum;
    m_fDimMin = fDimMin;
    m_fDimMax = fDimMax;
    m_iDimMinPos = iDimMinPos;
    m_iDimMaxPos = iDimMaxPos;
    m_fDimMean = fDimMean;
    
  }
  
  
  //////////////////////////////
  // SET METHODS              //
  //////////////////////////////
  public abstract void setValues(float[] pValues);




}