package listeners;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.image.BufferedImage;

/** Interface for all views that generate graphics that should be exported as images.
 */
public interface IExportable {

  /** Returns the current width of the view implementing this interface.
   *
   *  @return The current width of the view as integer.
   */
  public int getWidth();


  /** Returns the current height of the view implementing this interface.
   *
   *  @return The current height of the view as integer.
   */
  public int getHeight();


  /** Renders the view into the given BufferedImage that will be saved as image file.
   *
   *  @param bimg BufferedImage object on which the view should be rendered.
   */
  public void renderImage(BufferedImage pImage);

}