package tools;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */


import java.util.*;


public class DistanceCalculator {


  public static Vector<IndexAndValue> getMahalanobisDistances(
    float[][] pData, 
    float[][] pCovarianceMatrix, 
    float[] pMeanVector
  ) {
    //1. make some checks
    if (pData == null) return null;
    if (pCovarianceMatrix == null) return null;
    if (pMeanVector == null) return null;
    
    int iEntryNum = pData[0].length;
    Vector<IndexAndValue> pDistances = new Vector<IndexAndValue>();

    for (int i = 0; i < iEntryNum; i++) {
      pDistances.add(new IndexAndValue(i, 0.0f));
    }
    
    //TODO: bring this back into the program with a new math library
    /*
  
  
    //2. calculate inverse covariance matrix
    Matrix pMatrix = new Matrix(pCovarianceMatrix);
    Matrix pInversMatrix = pMatrix.inverse();
    float[][] pInverseCovarianceMatrix = pInversMatrix.getArrayCopyFloat();


    //3. calculate mahalanobis distance
    int iDimNum = pData.length;
    int iEntryNum = pData[0].length;
    Vector<IndexAndValue> pDistances = new Vector<IndexAndValue>();

    for (int i = 0; i < iEntryNum; i++) {

      //3.1.create meannormalized entry
      float[] pDataItem = new float[iDimNum];
      for (int dim = 0; dim < iDimNum; dim++) {
        pDataItem[dim] = pData[dim][i] - pMeanVector[dim];
      }

      //3.2. calculate the distance
      float fDistance = 0.0f;
      for (int iRow = 0; iRow < iDimNum; iRow++) {
        for (int iCol = 0; iCol < iDimNum; iCol++) {
          fDistance += pDataItem[iRow]*pInverseCovarianceMatrix[iRow][iCol]*pDataItem[iCol];
        }
      }

      fDistance = (float)Math.sqrt(fDistance);

      //3.3. add it to the vector
      pDistances.add(new IndexAndValue(i, fDistance));

      //System.out.println((entry+1) + ": " + distance);
    }
    */

    return pDistances;


  }

  
  public static float[] getMahalanobisDistancesAsFloatArray(
    float[][] pData, 
    float[][] pCovarianceMatrix, 
    float[] pMeanVector
  ) {
    
    //1. make some checks
    if (pData == null) return null;
    if (pCovarianceMatrix == null) return null;
    if (pMeanVector == null) return null;
  
    
    int iEntryNum = pData[0].length;
    float[] pDistances = new float[iEntryNum];

    for (int i = 0; i < iEntryNum; i++) {
      pDistances[i] = 0.0f;
    }
    
    //TODO: bring this back into the program with a new math library
    /*
    
  
    //2. calculate inverse covariance matrix
    Matrix pMatrix = new Matrix(pCovarianceMatrix);
    Matrix pInversMatrix = pMatrix.inverse();
    float[][] pInverseCovarianceMatrix = pInversMatrix.getArrayCopyFloat();
  
    


    //3. calculate mahalanobis distance
    int iDimNum = pData.length;
    int iEntryNum = pData[0].length;
    float[] pDistances = new float[iEntryNum];

    for (int i = 0; i < iEntryNum; i++) {

       //3.1.create meannormalized entry
      float[] pDataItem = new float[iDimNum];
      for (int dim = 0; dim < iDimNum; dim++) {
        pDataItem[dim] = pData[dim][i] - pMeanVector[dim];
      }

      //3.2. calculate the distance
      float fDistance = 0.0f;
      for (int iRow = 0; iRow < iDimNum; iRow++) {
        for (int iCol = 0; iCol < iDimNum; iCol++) {
          fDistance += pDataItem[iRow]*pInverseCovarianceMatrix[iRow][iCol]*pDataItem[iCol];
        }
      }

      fDistance = (float)Math.sqrt(fDistance);
      pDistances[i] = fDistance;

      //System.out.println((entry+1) + ": " + distance);
    }
    */

    return pDistances;


  }
  


}