package view;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */


import listeners.IDataSetListener;
import listeners.IAxisManagementListener;

import data.Data;


import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.border.TitledBorder;

import java.util.Vector;

public class AxisManagementFrame extends JFrame 
  implements WindowListener, ActionListener, ListSelectionListener, IDataSetListener {

  /** member variables **/
  private Container m_pContainer;

  /** axisManagement Listener **/
  private Vector<IAxisManagementListener> m_pAxisManagementListeners;





  /** **/
  private Data  m_pData;                          //holds the data
  private int   m_iDimNum;                        //holds the number of dimensions
  private JList m_pDimensionsList;                //list holding all dimension names
  private JCheckBox m_pVisibleCheckbox;           //indicates if the selected axis should be visible
  private JCheckBox m_pFlipCheckbox;              //indicates if the selected axis should be flipped
  private JCheckBox m_pShowDimNamesCheckbox;      //indicates if the dimension names are shown
  private JCheckBox m_pShowDimValuesCheckbox;      //indicates if the dimension names are shown


  /** axis dependent properties in arrays**/
  private boolean[] m_pFlip;        //boolean array indicating if axis i should be flipped
  private boolean[] m_pVisible;     //boolean array indicating if axis i should be shown
  private int[]     m_pOrder;       //integer array indicating the order of the dimensions

  ///////////////////////////////
  // CONSTRUCTOR               //
  ///////////////////////////////
  //setting member variables, creating GUI
  public AxisManagementFrame(Data pData) {
    //1. init the windows properties
    setResizable(true);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    addWindowListener(this);
    
    setSize(new Dimension(400, 300));
    setTitle("Axis management of parallel coordinates");

    //2. set Container-Properties
    m_pContainer = this.getContentPane();
    BoxLayout pBoxLayout = new BoxLayout(m_pContainer, BoxLayout.Y_AXIS);
    m_pContainer.setLayout(pBoxLayout);
    

    //3. set member variables
    m_pData = pData;
    m_iDimNum = m_pData.getDimNumber();
    m_pAxisManagementListeners = new Vector<IAxisManagementListener>();
    
    m_pFlip = new boolean[m_iDimNum];
    m_pVisible = new boolean[m_iDimNum];
    m_pOrder = new int[m_iDimNum];
    for (int i = 0; i < m_iDimNum; i++) {
      m_pFlip[i] = false;
      m_pVisible[i] = true;
      m_pOrder[i] = i;
    }


    //4. create the panels
    //create panel of the general properties
    JPanel pGeneralPropertiesPanel = createGeneralPropertiesPanel();

    //create panal for managing the individual axes
    JPanel pAxisPropertiesPanel = createAxisPropertiesPanel();

    //add the components to the panel
    m_pContainer.add(pGeneralPropertiesPanel);
    m_pContainer.add(pAxisPropertiesPanel);




  }


  ////////////////////////////////////
  // GET METHODS                    //
  ////////////////////////////////////
  public boolean areDimensionNamesVisible() {
    if (m_pShowDimNamesCheckbox == null) return false;
    return m_pShowDimNamesCheckbox.isSelected();
  }

  public boolean areExtremaVisible() {
    if (m_pShowDimValuesCheckbox == null) return false;
    return m_pShowDimValuesCheckbox.isSelected();
  }


  public int[] getVisibleAxisOrder() {

    //1. check how many axis are visible?
    int iCountVisible = 0;
    for (int i = 0; i < m_iDimNum; i++) {
      if (m_pVisible[i]) iCountVisible++;
    }

    //2. create the visible axis array containing the order of appearance
    int[] pVisibleAxisOrder = new int[iCountVisible];
    int iCounter = 0;

    for (int i = 0; i < m_iDimNum; i++) {
      if (m_pVisible[i]) {
        pVisibleAxisOrder[iCounter] = m_pOrder[i];
        iCounter++;
      }
    }

    return pVisibleAxisOrder;

  }


  //returns an array containing true for flipping a axis
  //only the visible axis are considered
  public boolean[] getFlippedAxis() {

    //1. check how many axis are visible?
    int iCountVisible = 0;
    for (int i = 0; i < m_iDimNum; i++) {
      if (m_pVisible[i]) iCountVisible++;
    }

    //2. create the flipped axis array containing only the IDs of the flipped axes
    boolean[] pFlippedAxis = new boolean[iCountVisible];
    int iCounter = 0;

    for (int i = 0; i < m_iDimNum; i++) {
      if (m_pVisible[i]) {
        pFlippedAxis[iCounter] = m_pFlip[i];
        iCounter++;
      }
    }

    return pFlippedAxis;


  }



  ////////////////////////////////////
  // CREATING JPANELS               //
  ////////////////////////////////////
  private JPanel createGeneralPropertiesPanel() {
    //1. create the panel and set the layout
    JPanel pGenPropsPanel = new JPanel();
    pGenPropsPanel.setLayout(new GridLayout(2,1));

    //2. create the border
    TitledBorder pGenPropsBorder = new TitledBorder("General axis properties");
    pGenPropsPanel.setBorder(pGenPropsBorder);

    //3. create checkboxes
    m_pShowDimNamesCheckbox = new JCheckBox("Show dimension names");
    m_pShowDimNamesCheckbox.setSelected(true);
    m_pShowDimNamesCheckbox.addActionListener(this);

    m_pShowDimValuesCheckbox = new JCheckBox("Show dimension minima and maxima");
    m_pShowDimValuesCheckbox.setSelected(true);
    m_pShowDimValuesCheckbox.addActionListener(this);


    //4. add the components to the panel
    pGenPropsPanel.add(m_pShowDimNamesCheckbox);
    pGenPropsPanel.add(m_pShowDimValuesCheckbox);

    return pGenPropsPanel;
  }



  private JPanel createAxisPropertiesPanel() {
    //1. create the panel and set the layout
    JPanel pAxisPropertiesPanel = new JPanel();
    pAxisPropertiesPanel.setLayout(new GridLayout(1,2, 10, 10));

    //2. create the border
    TitledBorder pAxisPropsBorder = new TitledBorder("Axis specific properties");
    pAxisPropertiesPanel.setBorder(pAxisPropsBorder);

    //3. as at the dimension weighting: a panel to select several dimensions
    //   Create a JList that displays the dimensions
    String[] pDimNames = m_pData.getDimensionNames();

    m_pDimensionsList = new JList(pDimNames);
    m_pDimensionsList.addListSelectionListener(this);
    m_pDimensionsList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

    JScrollPane pScrollPane = new JScrollPane(m_pDimensionsList);
    pScrollPane.setPreferredSize(new Dimension(200, 100));
    pScrollPane.setMinimumSize(new Dimension(200, 100));

    JPanel pSubPanel = createAxisPropertiesSubPanel();


    //4. add components
    pAxisPropertiesPanel.add(pScrollPane);
    pAxisPropertiesPanel.add(pSubPanel);


    return pAxisPropertiesPanel;
  }


  private JPanel createAxisPropertiesSubPanel() {

    //1. create the panel and set the layout
    JPanel pSubPanel = new JPanel();
    pSubPanel.setLayout(new GridLayout(5,1, 10, 10));


    //2. set Axis visible
    m_pVisibleCheckbox = new JCheckBox("Show selected axis");
    m_pVisibleCheckbox.setSelected(true);
    m_pVisibleCheckbox.setEnabled(false);
    m_pVisibleCheckbox.addActionListener(this);

    //3. flip Axis
    m_pFlipCheckbox = new JCheckBox("Flip selected axis");
    m_pFlipCheckbox.setSelected(true);
    m_pFlipCheckbox.setEnabled(false);
    m_pFlipCheckbox.addActionListener(this);


    //4. move selected axis up
    JButton pMoveAxisUp = new JButton("Move selected axis up");
    pMoveAxisUp.addActionListener(this);

    //5. move selected axis down
    JButton pMoveAxisDown = new JButton("Move selected axis down");
    pMoveAxisDown.addActionListener(this);


    //6. set all Axis visible
    JButton pAllAxis = new JButton("Show all axis");
    pAllAxis.addActionListener(this);


    //7. add components
    pSubPanel.add(m_pVisibleCheckbox);
    pSubPanel.add(m_pFlipCheckbox);

    pSubPanel.add(pMoveAxisUp);
    pSubPanel.add(pMoveAxisDown);

    pSubPanel.add(pAllAxis);



    return pSubPanel;


  }



  //////////////////////////////////////////
  // AXISMANAGEMENT LISTENER METHODS      //
  //////////////////////////////////////////
  public void addAxisManagementListener(IAxisManagementListener pListener) {
    if (pListener == null) return;
    
    m_pAxisManagementListeners.add(pListener);
  }

    
  private void axisManagementChanged() {
    int iListenerNum = m_pAxisManagementListeners.size();
    for (int i = 0; i < iListenerNum; i++) {
      IAxisManagementListener pListener = m_pAxisManagementListeners.get(i);
      pListener.axisManagementChanged(this);
    }
  }





  ////////////////////////////////////
  // IDATASETLISTENER               //
  ////////////////////////////////////

  public void dimensionValuesChanged(Data pDataset, String pDimName) {
  }

  public void dimensionNumberChanged(Data pDataset, String pDimName) {

      //1. create new array of flipped dimensions (it is only designed for one more dimension)
      int iNewArrayLength = m_pFlip.length+1;
      boolean[] pNewFlipArray = new boolean[iNewArrayLength];
      for (int i = 0; i < iNewArrayLength; i++) {

        if (i < iNewArrayLength-1) {
          pNewFlipArray[i] = m_pFlip[i];
        }
        else {
          pNewFlipArray[i] = false;
        }
      }
      m_pFlip = pNewFlipArray;

      
      //2. create new visibility array
      iNewArrayLength = m_pVisible.length+1;
      boolean[] pNewVisibilityArray = new boolean[iNewArrayLength];
      for (int i = 0; i < iNewArrayLength; i++) {

        if (i < iNewArrayLength-1) {
          pNewVisibilityArray[i] = m_pVisible[i];
        }
        else {
          pNewVisibilityArray[i] = false;
        }
      }
      m_pVisible = pNewVisibilityArray;


      //3. create new order array
      iNewArrayLength = m_pOrder.length+1;
      int[] pNewOrderArray = new int[iNewArrayLength];
      for (int i = 0; i < iNewArrayLength; i++) {

        if (i < iNewArrayLength-1) {
          pNewOrderArray[i] = m_pOrder[i];
        }
        else {
          pNewOrderArray[i] = i;
        }
      }
      m_pOrder = pNewOrderArray;



      //4. add the dimension names according their order to the list
      String[] pDimNames = m_pData.getDimensionNames();
      int iDimNamesNum = pDimNames.length;
      String[] pOrderedDimNames = new String[iDimNamesNum];
      for (int i = 0; i < iDimNamesNum; i++) {
        pOrderedDimNames[i] = pDimNames[m_pOrder[i]];
      }
      m_pDimensionsList.setListData(pOrderedDimNames);
      
      //5. update the dimension selection list
      int[] pSelectedIndices = m_pDimensionsList.getSelectedIndices();
      m_pDimensionsList.setSelectedIndices(pSelectedIndices);

      m_iDimNum++;


  }


  ////////////////////////////////////
  // WINDOWLISTENER                 //
  ////////////////////////////////////
  public void windowClosing(WindowEvent e) {
    setVisible(false);
  }
  public void windowGainedFocus(WindowEvent e) {}
  public void windowClosed(WindowEvent e) {}
  public void windowDeactivated(WindowEvent e) {}
  public void windowActivated(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}
  public void windowOpened(WindowEvent e) {}



  ////////////////////////////////////
  // ACTIONLISTENER                 //
  ////////////////////////////////////
  public void actionPerformed(ActionEvent e) {

    //1. set the visibility of the selected axis to the value of the visibility checkbox
    if (e.getActionCommand().equals("Show selected axis")) {


      int[] pSelectedIndices = m_pDimensionsList.getSelectedIndices();
      int iIndicesNum = pSelectedIndices.length;
      
      for (int i = 0; i < iIndicesNum; i++) {
        m_pVisible[pSelectedIndices[i]] = m_pVisibleCheckbox.isSelected();
      }
    }

    //2. set flip-flag of the selected axis to the value of the flip checkbox
    if (e.getActionCommand().equals("Flip selected axis")) {
      int[] pSelectedIndices = m_pDimensionsList.getSelectedIndices();
      int iIndicesNum = pSelectedIndices.length;
      
      for (int i = 0; i < iIndicesNum; i++) {
        m_pFlip[pSelectedIndices[i]] = m_pFlipCheckbox.isSelected();
      }
    }


    //3. move selected axis up
    if (e.getActionCommand().equals("Move selected axis up")) {
      int[] pSelectedIndices = m_pDimensionsList.getSelectedIndices();
      int iIndicesNum = pSelectedIndices.length;

      //3.1. skip the operation if no dim selected or the first dim is selected
      if (iIndicesNum == 0) return;
      if (pSelectedIndices[0] == 0) return;


      //3.2. change the order of the dimIDs in the m_pOrder array and the flags in the
      //visibility and flip array
      for (int i = 0; i < iIndicesNum; i++) {
        int iCurrIndex = pSelectedIndices[i];

        //dimID
        int iHelp = m_pOrder[iCurrIndex];
        m_pOrder[iCurrIndex] = m_pOrder[iCurrIndex-1];
        m_pOrder[iCurrIndex-1] = iHelp;

        //visibility
        boolean bHelp = m_pVisible[iCurrIndex];
        m_pVisible[iCurrIndex] = m_pVisible[iCurrIndex-1];
        m_pVisible[iCurrIndex-1] = bHelp;

        //flip
        bHelp = m_pFlip[iCurrIndex];
        m_pFlip[iCurrIndex] = m_pFlip[iCurrIndex-1];
        m_pFlip[iCurrIndex-1] = bHelp;

      }

      //3.3. add the dimension names according their order to the list
      String[] pDimNames = m_pData.getDimensionNames();
      int iDimNameNum = pDimNames.length;
      String[] pOrderedDimNames = new String[iDimNameNum];

      for (int i = 0; i < iDimNameNum; i++) {
        pOrderedDimNames[i] = pDimNames[m_pOrder[i]];
      }

      m_pDimensionsList.setListData(pOrderedDimNames);
      for (int i = 0; i < iIndicesNum; i++) {
        pSelectedIndices[i]--;
      }
      m_pDimensionsList.setSelectedIndices(pSelectedIndices);

    }


    //4. move selected axis down
    if (e.getActionCommand().equals("Move selected axis down")) {
      int[] pSelectedIndices = m_pDimensionsList.getSelectedIndices();
      int iIndicesNum = pSelectedIndices.length;

      //4.1. skip the operation if no dim selected or the last dim is selected
      if (iIndicesNum == 0) return;
      if (pSelectedIndices[iIndicesNum-1] == m_pOrder.length-1) return;


      //4.2. change the order of the dimIDs in the m_pOrder array and the flags in the
      //     visibility and flip array
      for (int i = 0; i < iIndicesNum; i++) {
        int iCurrIndex = pSelectedIndices[i];

        //dimID
        int iHelp = m_pOrder[iCurrIndex];
        m_pOrder[iCurrIndex] = m_pOrder[iCurrIndex+1];
        m_pOrder[iCurrIndex+1] = iHelp;

        //visibility
        boolean bHelp = m_pVisible[iCurrIndex];
        m_pVisible[iCurrIndex] = m_pVisible[iCurrIndex+1];
        m_pVisible[iCurrIndex+1] = bHelp;

        //flip
        bHelp = m_pFlip[iCurrIndex];
        m_pFlip[iCurrIndex] = m_pFlip[iCurrIndex+1];
        m_pFlip[iCurrIndex+1] = bHelp;

      }

      //4.3. add the dimension names according their order to the list
      String[] pDimNames = m_pData.getDimensionNames();
      int iDimNameNum = pDimNames.length;
      String[] pOrderedDimNames = new String[iDimNameNum];
      
      for (int i = 0; i < iDimNameNum; i++) {
        pOrderedDimNames[i] = pDimNames[m_pOrder[i]];
      }

      m_pDimensionsList.setListData(pOrderedDimNames);
      for (int i = 0; i < iIndicesNum; i++) {
        pSelectedIndices[i]++;
      }
      m_pDimensionsList.setSelectedIndices(pSelectedIndices);



    }





    //5. set all axis visible
    if (e.getActionCommand().equals("Show all axis")) {
      for (int i = 0; i < m_iDimNum; i++) {
        m_pVisible[i] = true;
      }
    }

    //6. tell the axesmanagement listeners that something changed
    axisManagementChanged();
  }

  ////////////////////////////////////
  // LISTSELECTIONLISTENER          //
  ////////////////////////////////////
  public void valueChanged(ListSelectionEvent e) {
    int[] pSelectedIndices = m_pDimensionsList.getSelectedIndices();
    int iIndicesNum = pSelectedIndices.length;
    if (iIndicesNum == 0) {
      m_pFlipCheckbox.setEnabled(false);
      m_pVisibleCheckbox.setEnabled(false);
      return;
    }

    m_pFlipCheckbox.setEnabled(true);
    m_pVisibleCheckbox.setEnabled(true);


    //calculate visibility and flip status
    //0 = not selected, 1 = selected, -1 = both happens
    //int iFlipStatus = 0;
    //int iVisibilityStatus = 0;



    //flip
    if (m_pFlip[pSelectedIndices[0]]) {
      m_pFlipCheckbox.setSelected(true);
    }
    else {
      m_pFlipCheckbox.setSelected(false);
    }

    //visibility
    if (m_pVisible[pSelectedIndices[0]]) {
      m_pVisibleCheckbox.setSelected(true);
    }
    else {
      m_pVisibleCheckbox.setSelected(false);
    }


  }



}