package view;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */


import listeners.IColorPanelListener;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;


public class ColorPanel extends JPanel implements MouseListener {

  /*** member variables ***/
  private Color m_pColor;
  private Vector<IColorPanelListener> m_pColorPanelListeners;


  /** constructor **/
  public ColorPanel (Color pColor) {
    //1. add mouselistener functionality
    addMouseListener(this);

    //2. set the members
    if (pColor == null) pColor = Color.BLACK;
    m_pColor = pColor;
    m_pColorPanelListeners = new Vector<IColorPanelListener>();
    
    //3. repaint the panel
    repaint();

    
  }




  ////////////////////////////////////
  // GET, SET METHODS               //
  ////////////////////////////////////

  public void setColor (Color pColor) {
    //1. make a simple check
    if (pColor == null) return;
  
    //2. set the color and repaint the panel
    m_pColor = pColor;
    repaint();
  }

  
  public Color getColor() {
    return m_pColor;
  }



  ////////////////////////////////////
  // PAINTING                       //
  ////////////////////////////////////


  //calls the paint-method
  public void paintComponent(Graphics pGraphics) {
    super.paintComponent(pGraphics);
    update(pGraphics);
  }

  //paint-method
  public void update(Graphics pGraphics) {
    pGraphics.setColor(m_pColor);
    pGraphics.fillRect(0,0, getWidth(), getHeight());
  }


  ////////////////////////////////////
  // MOUSE LISTENER                 //
  ////////////////////////////////////

  //open the color chooser on double click or right mouse click
  public void mouseClicked(MouseEvent e) {
    if ((e.getClickCount() == 2) || (e.getButton() == MouseEvent.BUTTON3)) {
      Color pNewColor = JColorChooser.showDialog(this, "Choose new color", m_pColor);

      if (pNewColor != null) m_pColor = pNewColor;

      repaint();

      colorPanelChanged();
    }

  }



  public void mouseEntered(MouseEvent e) {}
  public void mouseExited(MouseEvent e)  {}
  public void mousePressed(MouseEvent e)  {}
  public void mouseReleased(MouseEvent e) {}



  //////////////////////////////////////////
  // COLORPANEL LISTENER METHODS          //
  //////////////////////////////////////////
  public void addColorPanelListener(IColorPanelListener pListener) {
    //1. make a simple check
    if (pListener == null) return;
    
    //2. add the listener
    m_pColorPanelListeners.add(pListener);
  }

  
  private void colorPanelChanged() {
    int iListenerNum = m_pColorPanelListeners.size();
    
    for (int i = 0; i < iListenerNum; i++) {
      IColorPanelListener pListener = m_pColorPanelListeners.get(i);
      pListener.colorPanelChanged(this);
    }

  }


}