package view;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.Color;

public class ColorPosition implements Comparable<ColorPosition> {

  /*** constants ***/
  public static final float MIN_POSITION = 0.0f;
  public static final float MAX_POSITION = 1.0f;

  /*** member variables ***/
  private Color m_pColor;
  private float m_fPosition;
  private ColorPosition m_pLowerNeighbour, m_pUpperNeighbour;


  /** constructor I**/
  public ColorPosition (Color pColor, float fPosition) {

    //1. set the members
    if (pColor == null) pColor = Color.BLACK;
    m_pColor = pColor;

    if (fPosition < MIN_POSITION) fPosition = MIN_POSITION;
    if (fPosition > MAX_POSITION) fPosition = MAX_POSITION;
    m_fPosition = fPosition;

    m_pLowerNeighbour = null;
    m_pUpperNeighbour = null;


  }

  /** constructor II**/
  public ColorPosition (ColorPosition pColorPosition) {
    if (pColorPosition == null) {
      m_pColor = Color.BLACK;
      m_fPosition = MIN_POSITION;
    }
    else {
      m_pColor = pColorPosition.getColor();
      m_fPosition = pColorPosition.getPosition();
    }
  }



  ///////////////////////////////////
  // SET METHODS                   //
  ///////////////////////////////////
  public void setColor(Color pColor) {
    //1. make a simple check
    if (pColor == null) return;
    
    //2. set the value
    m_pColor = pColor;
  }

  public void setPosition(float fPosition) {
    //1. make some checks
    if (fPosition < MIN_POSITION) fPosition = MIN_POSITION;
    if (fPosition > MAX_POSITION) fPosition = MAX_POSITION;

    //2. check lower neighbour
    if (m_pLowerNeighbour != null) {
      if (fPosition < m_pLowerNeighbour.getPosition()) {
        fPosition = m_pLowerNeighbour.getPosition();
      }
    }

    //3. check the upper neighbour
    if (m_pUpperNeighbour != null) {
      if (fPosition > m_pUpperNeighbour.getPosition()) {
        fPosition = m_pUpperNeighbour.getPosition();
      }
    }

    //4. set the position
    m_fPosition = fPosition;
  }

  
  public void setNeighbours(ColorPosition pLowerNeighbour, ColorPosition pUpperNeighbour) {
    //1. make some checks
    if ((pLowerNeighbour != null) && (pUpperNeighbour != null)) {
      if (pLowerNeighbour.getPosition() >= pUpperNeighbour.getPosition()) return;
    }

    //2. set the members
    m_pLowerNeighbour = pLowerNeighbour;
    m_pUpperNeighbour = pUpperNeighbour;
  }

  ///////////////////////////////////
  // GET METHODS                   //
  ///////////////////////////////////
  public Color getColor() {
    return m_pColor;
  }

  public float getPosition() {
    return m_fPosition;
  }


  /////////////////////////////////////
  // COMPARABLE INTERFACE            //
  /////////////////////////////////////
  /** method of the comparable interface **/
  public int compareTo(ColorPosition pColorPosition) {
    //1. make some checks
    if (pColorPosition == null) return 0;
  
    try {
      if (m_fPosition < pColorPosition.getPosition()) return -1;
      if (m_fPosition > pColorPosition.getPosition()) return 1;
      return 0;
    }
    catch (Exception e) {
      return 0;
    }

  }




}