package view;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import data.Data;
import data.DataDimension;
import data.FloatDataDimension;
import listeners.GlobalConstants;
import tools.CovarianceCalculator;
import tools.DistanceCalculator;

import javax.swing.JFrame;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.TitledBorder;


//frame for controlling the calculation of the robust and the mahalanobis distance
public class DistanceFrame extends JFrame implements WindowListener, ActionListener,
ChangeListener, ListSelectionListener, GlobalConstants {

  //member variables
  /*** GUI elements  ***/
  private Container m_pContainer;           //container for gui
  private JRadioButton m_pClassicDistance, m_pRobustDistance;
  private JSlider m_pHValueSlider;
  private JTextField m_pHValueTextField;
  private JButton m_pSaveDistanceButton;
  private JTextField m_pSaveNameTextField;

  /*** data elements ***/
  private Data      m_pData;           //data object
  private JList     m_pDimensionsList;   //list holding the relevant dimensions for PCA


  ///////////////////////////////////
  // CONSTRUCTOR                   //
  ///////////////////////////////////
  //Constructor, Creating of GUI
  public DistanceFrame(Data pData) {

    //1. set member variables
    m_pData = pData;


    //2. Window-Properties and listener functionalitys
    setSize(new Dimension(650, 450));
    this.setTitle("Distances");

    setResizable(true);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);

    addWindowListener(this);


    //3. Containers-Properties
    m_pContainer = getContentPane();
    
    GridBagLayout pGridBagLayout = new GridBagLayout();
    GridBagConstraints pGridBagConstraints = new GridBagConstraints();
    m_pContainer.setLayout(pGridBagLayout);
    

    //4. create the panels
    JPanel pDistType = createDistTypePanel();
    JPanel pRobustParams = createRobustParamsPanel();
    JPanel pDistanceParams = createDistanceParamsPanel();
    JPanel pButtons = createButtonPanel();


    //5. add components to container
    pGridBagConstraints.fill = GridBagConstraints.BOTH;
    pGridBagConstraints.weightx = 1.0;
    pGridBagConstraints.ipadx = 50;
    pGridBagConstraints.insets = new Insets(5, 5, 5, 5);

    pGridBagConstraints.gridwidth = GridBagConstraints.REMAINDER; //end row
    pGridBagLayout.setConstraints(pDistType, pGridBagConstraints);
    m_pContainer.add(pDistType);

    pGridBagLayout.setConstraints(pRobustParams, pGridBagConstraints);
    m_pContainer.add(pRobustParams);

    pGridBagLayout.setConstraints(pDistanceParams, pGridBagConstraints);
    m_pContainer.add(pDistanceParams);

    pGridBagLayout.setConstraints(pButtons, pGridBagConstraints);
    m_pContainer.add(pButtons);

  }


  ////////////////////////////////////
  // GUI CREATION METHODS           //
  ////////////////////////////////////

  private JPanel createDistTypePanel() {
    //1. create the panel and the layout
    JPanel pDistTypePanel = new JPanel();
    pDistTypePanel.setLayout(new FlowLayout());


    //2. create the border
    TitledBorder pDistTypePanelBorder = new TitledBorder("Type of distance");
    pDistTypePanel.setBorder(pDistTypePanelBorder);


    //3. create radiobuttons
    m_pClassicDistance = new JRadioButton("Mahalanobis distance");
    m_pClassicDistance.setSelected(true);
    m_pClassicDistance.addActionListener(this);

    m_pRobustDistance = new JRadioButton("Robust distance");
    m_pRobustDistance.addActionListener(this);


    //4. add the buttons to a button group
    ButtonGroup pTypeButtonGroup = new ButtonGroup();
    pTypeButtonGroup.add(m_pClassicDistance);
    pTypeButtonGroup.add(m_pRobustDistance);

    //5. add the components to the panel
    pDistTypePanel.add(m_pClassicDistance);
    pDistTypePanel.add(m_pRobustDistance);

    return pDistTypePanel;
  }

  private JPanel createRobustParamsPanel() {
    //1. create the panel and the layout
    JPanel pRobustParamsPanel = new JPanel();
    GridBagLayout pGridBagLayout = new GridBagLayout();
    GridBagConstraints pGridBagConstraints = new GridBagConstraints();
    pRobustParamsPanel.setLayout(pGridBagLayout);


    //2. create the border
    TitledBorder pRobustParamsPanelBorder = new TitledBorder("Robustness parameter");
    pRobustParamsPanel.setBorder(pRobustParamsPanelBorder);


    //3. ROBUST PARAMETER H (which percentage of the data)
    //3.1. labels
    JLabel pHLabel = new JLabel("Percentage of data (h) for robust estimate");
    JLabel pHMinLabel = new JLabel("50 %");
    JLabel pHMaxLabel = new JLabel("100 %");
    pHMaxLabel.setHorizontalAlignment(JLabel.RIGHT);


    //3.2. JSlider
    m_pHValueSlider = new JSlider(50, 100, 50);
    m_pHValueSlider.setEnabled(false);
    m_pHValueSlider.addChangeListener(this);

    //3.3. TextField
    m_pHValueTextField = new JTextField("50 %");
    m_pHValueTextField.setHorizontalAlignment(JTextField.RIGHT);
    m_pHValueTextField.setEnabled(false);
    m_pHValueTextField.addActionListener(this);
    m_pHValueTextField.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        hValueTextFieldActionPerformed(e);
      }
    });


    //4. add the components to the panel
    pGridBagConstraints.fill = GridBagConstraints.BOTH;
    pGridBagConstraints.weightx = 1.0;
    pGridBagConstraints.ipadx = 50;
    pGridBagConstraints.insets = new Insets(5, 5, 5, 5);

    pGridBagConstraints.gridwidth = GridBagConstraints.REMAINDER; //end row
    pGridBagLayout.setConstraints(pHLabel, pGridBagConstraints);
    pRobustParamsPanel.add(pHLabel);

    pGridBagConstraints.gridwidth = GridBagConstraints.REMAINDER; //end row
    pGridBagLayout.setConstraints(m_pHValueSlider, pGridBagConstraints);
    pRobustParamsPanel.add(m_pHValueSlider);

    pGridBagConstraints.gridwidth = 1;
    pGridBagConstraints.weightx = 1;
    pGridBagLayout.setConstraints(pHMinLabel, pGridBagConstraints);
    pRobustParamsPanel.add(pHMinLabel);

    pGridBagConstraints.weightx = 0;
    pGridBagLayout.setConstraints(m_pHValueTextField, pGridBagConstraints);
    pRobustParamsPanel.add(m_pHValueTextField);


    pGridBagConstraints.weightx = 1;
    pGridBagConstraints.gridwidth = GridBagConstraints.REMAINDER; //end row
    pGridBagLayout.setConstraints(pHMaxLabel, pGridBagConstraints);
    pRobustParamsPanel.add(pHMaxLabel);

    return pRobustParamsPanel;
  }



  private JPanel createDistanceParamsPanel() {
    //1. create the panel and the layout
    JPanel pDistanceParamsPanel = new JPanel();
    pDistanceParamsPanel.setLayout(new BoxLayout(pDistanceParamsPanel, BoxLayout.X_AXIS));

    //2. create the border
    TitledBorder pDistanceParamsPanelBorder = new TitledBorder("PCA parameters");
    pDistanceParamsPanel.setBorder(pDistanceParamsPanelBorder);


    //3. Create a JList that displays the dimensions
    String[] pDimNames = m_pData.getDimensionNames();

    //delete DOI from list if the list contains it
    int iDimNum = pDimNames.length;
    for (int i = 0; i < iDimNum; i++) {
      //if a dimension has the name of a DOI dimension -> we have to delete it
      if (pDimNames[i].equals(DOINAME)) {
        int iNewDimNum = iDimNum-1;
        String[] pNewDimNames = new String[iNewDimNum];
        int iAddCount = 0;
        for (int j = 0; j < iNewDimNum; j++) {
          if (j == i) {
            iAddCount = 1;
          }
          pNewDimNames[j] = pDimNames[j+iAddCount];
        }
        pDimNames = pNewDimNames;
        break;
      }
    }
    m_pDimensionsList = new JList(pDimNames);
    m_pDimensionsList.addListSelectionListener(this);

    JScrollPane pScrollPane = new JScrollPane(m_pDimensionsList);
    pScrollPane.setPreferredSize(new Dimension(200, 100));
    pScrollPane.setMinimumSize(new Dimension(200, 100));
    pScrollPane.setMaximumSize(new Dimension(200, 100));



    //4. create subpanel
    JPanel pDistanceParamsSubPanel = createDistanceParamsSubPanel();
    pDistanceParamsSubPanel.setPreferredSize(new Dimension(40, 100));
    pDistanceParamsSubPanel.setMinimumSize(new Dimension(40, 100));


    //5. add components to the panel
    pDistanceParamsPanel.add(pScrollPane);
    pDistanceParamsPanel.add(new JLabel("       "));
    pDistanceParamsPanel.add(pDistanceParamsSubPanel);

    //6. return the panel
    return pDistanceParamsPanel;
  }


  private JPanel createDistanceParamsSubPanel() {
    //1. create the panel and the layout
    JPanel pPCAParamsSubPanel = new JPanel();
    pPCAParamsSubPanel.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
    GridBagLayout pGridBagLayout = new GridBagLayout();
    GridBagConstraints pGridBagConstraints = new GridBagConstraints();
    pPCAParamsSubPanel.setLayout(pGridBagLayout);


    //2. Label
    JLabel pSaveLabel = new JLabel("Name for distance channel: ");

    //3. TextField
    m_pSaveNameTextField = new JTextField("Mahalanobis distance");


    //4. add the elements to the panel
    pGridBagConstraints.fill = GridBagConstraints.BOTH;
    pGridBagConstraints.weightx = 1.0;
    pGridBagConstraints.ipadx = 50;
    pGridBagConstraints.insets = new Insets(5, 5, 5, 5);

    pGridBagConstraints.gridwidth = 2; //end row
    pGridBagConstraints.weightx = 1.0;
    pGridBagLayout.setConstraints(m_pSaveNameTextField, pGridBagConstraints);
    pPCAParamsSubPanel.add(m_pSaveNameTextField);

    pGridBagConstraints.gridwidth = GridBagConstraints.REMAINDER; //end row
    pGridBagConstraints.weightx = 0;
    pGridBagLayout.setConstraints(pSaveLabel, pGridBagConstraints);
    pPCAParamsSubPanel.add(pSaveLabel);


    return pPCAParamsSubPanel;
  }




  private JPanel createButtonPanel() {

    //1. create the panel and the layout
    JPanel pButtonsPanel = new JPanel();
    pButtonsPanel.setLayout(new FlowLayout());

    //2. buttons
    m_pSaveDistanceButton = new JButton("Add mahalanobis distance to data");
    m_pSaveDistanceButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        addDistanceToData();
      }
    });

    //3. add components to panel
    pButtonsPanel.add(m_pSaveDistanceButton);

    return pButtonsPanel;
  }

  ////////////////////////////////////////
  //ACTIONLISTENER METHODS              //
  ////////////////////////////////////////
  public void actionPerformed(ActionEvent e) {

    //1. check if robust pca is active
    if (m_pRobustDistance.isSelected()) {
      m_pHValueSlider.setEnabled(true);
      m_pHValueTextField.setEnabled(true);

      m_pSaveDistanceButton.setText("Add robust distance to data");

      if (m_pSaveNameTextField.getText().equals("Mahalanobis distance")) {
        m_pSaveNameTextField.setText("Robust distance");
      }
    }

    //2. if the classic pca is active
    else {
      m_pHValueSlider.setEnabled(false);
      m_pHValueTextField.setEnabled(false);
      m_pSaveDistanceButton.setText("Add mahalanobis distance to data");

      if (m_pSaveNameTextField.getText().equals("Robust distance")) {
        m_pSaveNameTextField.setText("Mahalanobis distance");
      }
    }

  }

  //action of changing the value in the robustness textfield
  private void hValueTextFieldActionPerformed(ActionEvent e) {
    try {
      int iValue = Integer.parseInt(m_pHValueTextField.getText());

      if (iValue < 50) {
        iValue = 50;
        m_pHValueTextField.setText("50 %");
      }

      if (iValue > 100) {
        iValue = 100;
        m_pHValueTextField.setText("100 %");
      }

      m_pHValueSlider.setValue(iValue);
    }
    catch(Exception pException) {
      m_pHValueTextField.setText(m_pHValueSlider.getValue() + " %");
    }

  }

  ////////////////////////////////////
  // LISTSELECTIONLISTENER          //
  ////////////////////////////////////
  public void valueChanged(ListSelectionEvent e) {
    /*int[] selected = m_pDimensionsList.getSelectedIndices();

    //set entries in used PCs choice
    int choiceSelection = m_chUsedPCs.getSelectedIndex();
    m_chUsedPCs.removeAll();
    if (selected.length > 0) {
      for (int i = 0; i < selected.length; i++) {
        m_chUsedPCs.add((i+1)+"");
      }

      if (choiceSelection == -1) {
        m_chUsedPCs.select(0);
      }
      else {
        if (choiceSelection > m_chUsedPCs.getItemCount()-1) {
          choiceSelection = m_chUsedPCs.getItemCount()-1;
        }
        m_chUsedPCs.select(choiceSelection);


      }
    }


    //check when if we have to recalculate the pca
    if (selected.length == 1) {
      calculatePCA(true);
      return;
    }

    if (m_PCA == null) {
      calculatePCA(true);
    }
    else {
      if (m_PCA.getPrincipalComponentNumber() != selected.length) {
        calculatePCA(true);
      }
    }*/


  }



  ////////////////////////////////////////
  //CHANGELISTENER METHOD               //
  ////////////////////////////////////////
  public void stateChanged(ChangeEvent e) {
    m_pHValueTextField.setText(m_pHValueSlider.getValue() + " %");
  }


  ////////////////////////////////////////
  //WindowsListener methods             //
  ////////////////////////////////////////
  public void windowClosing(WindowEvent event1) {
    setVisible(false);
  }
  public void windowGainedFocus(WindowEvent event2) {}
  public void windowClosed(WindowEvent e) {}
  public void windowDeactivated(WindowEvent e) {}
  public void windowActivated(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}
  public void windowOpened(WindowEvent e) {}





  ////////////////////////////////////
  // DISTANCE FUNCTIONALITY         //
  ////////////////////////////////////

  private void addDistanceToData() {


    //1. check errors
    if (m_pDimensionsList.getSelectedIndices().length <= 1) {
      String pTitle = "Error";
      String pMessage = "At least 2 dimensions for calculating the location and spread of the data have to be selected.";
      JOptionPane.showMessageDialog(this, pMessage, pTitle, JOptionPane.ERROR_MESSAGE);
      return;
    }

    if (m_pSaveNameTextField.getText().equals("")) {
      String pTitle = "Error";
      String pMessage = "No name for the distance dimension is specified.";
      JOptionPane.showMessageDialog(this, pMessage, pTitle, JOptionPane.ERROR_MESSAGE);
      return;
    }


    //2. create a copy of the data needed
    Object[] pDimensions = m_pDimensionsList.getSelectedValues();
    int iDimNum = pDimensions.length;
    int iEntryNum = m_pData.getEntryNum();

    float[][] pDataArray = new float[iDimNum][iEntryNum];

    for (int i = 0; i < iDimNum; i++) {
      String pDimName = (String)pDimensions[i];
      pDataArray[i] = m_pData.getDimensionValues(pDimName);
    }



    //3. calculate covariance matrix (spread)
    float[][] pCovarianceMatrix;
    CovarianceCalculator pCovarianceCalculator = new CovarianceCalculator();

    if (m_pClassicDistance.isSelected()) {
      pCovarianceMatrix = pCovarianceCalculator.classicCovarianceMatrix(pDataArray);
    }
    else {
      float fHValue = (float)m_pHValueSlider.getValue()/100.0f;
      pCovarianceMatrix = pCovarianceCalculator.robustCovarianceMatrix(fHValue, pDataArray);
    }

    //4. get the mean (location)
    float[] pMeanVector = pCovarianceCalculator.getMeanVector();


    //5. calculate the distance
    float[] pDistances = DistanceCalculator.getMahalanobisDistancesAsFloatArray(pDataArray, pCovarianceMatrix, pMeanVector);



    //6. save the distances as dimension in the data
    DataDimension pDistDim = new FloatDataDimension(pDistances, m_pSaveNameTextField.getText());
    m_pData.addDimension(pDistDim);


    //7. say user that everything is ok
    String pTitle = "Distances saved.";
    String pMessage = "The distances have been added to the dataset.";
    JOptionPane.showMessageDialog(this, pMessage, pTitle, JOptionPane.INFORMATION_MESSAGE);



  }


}