package view;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import listeners.IDataSubsetListener;

import java.awt.event.MouseEvent;
import java.util.Vector;

public abstract class Interaction {

  private Vector<IDataSubsetListener> m_pListeners;


  Interaction(Vector<IDataSubsetListener> pListeners) {
    m_pListeners = pListeners;
  }


  ////////////////////////////////////////
  //IDataSubsetListener methods         //
  ////////////////////////////////////////

  /** Method that is called from within the Selection implementation to notify all registered
   *  listeners if changes have occured.
   *
   */
  protected void setSelectedDataAsData() {
  
    int iListenerNum = m_pListeners.size();
    for (int i = 0; i < iListenerNum; i++) {
      IDataSubsetListener pListener = m_pListeners.get(i);
      pListener.setSelectedDataAsData();
    }

  }

  protected void setSelectedDataAsData(float[] pDOI) {
    int iListenerNum = m_pListeners.size();
    for (int i = 0; i < iListenerNum; i++) {
      IDataSubsetListener pListener = m_pListeners.get(i);
      pListener.setSelectedDataAsData(pDOI);
    }

  }



  protected void resetWholeDatasetAsData() {
    int iListenerNum = m_pListeners.size();
    for (int i = 0; i < iListenerNum; i++) {
      IDataSubsetListener pListener = m_pListeners.get(i);
      pListener.resetWholeDatasetAsData();
    }

  }



  /** indicates if the CTRL-key is pressed **/
  public abstract void setControlPressed(boolean pressed);

  ////////////////////////////////////////
  //MouseListener methods               //
  ////////////////////////////////////////
  public abstract void mouseClicked(MouseEvent e);
  public abstract void mouseEntered(MouseEvent e);
  public abstract void mouseExited(MouseEvent e);
  public abstract void mousePressed(MouseEvent e);
  public abstract void mouseReleased(MouseEvent e);

  ////////////////////////////////////////
  //MouseMotionListener methods         //
  ////////////////////////////////////////
  public abstract void mouseDragged(MouseEvent e);
  public abstract void mouseMoved(MouseEvent e);


}