package view;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import javax.swing.*;


public class MyCellRenderer extends JLabel implements ListCellRenderer {

  /** member variables ***/
  private int[] m_pWeightingValues;

     
  //////////////////////////
  // CONSTRUCTOR          //
  //////////////////////////
  public MyCellRenderer(int iItemNumber) {
    m_pWeightingValues = new int[iItemNumber];

    for (int i = 0; i < iItemNumber; i++) {
      m_pWeightingValues[i] = 255;
    }
  }

     
  //////////////////////////
  // SET METHODS          //
  //////////////////////////
  public void setAllWeightsToZero() {
    int iItemNumber = m_pWeightingValues.length;
    for (int i = 0; i < iItemNumber; i++) {
      m_pWeightingValues[i] = 0;
    }
  }

  public void setAllWeightsToOne() {
    int iItemNumber = m_pWeightingValues.length;
  
    for (int i = 0; i < iItemNumber; i++) {
      m_pWeightingValues[i] = 255;
    }
  }

  
  public void setWeight(int iItemId, int iValue) {
    //1. make some checks
    if ((iItemId < 0) || (iItemId >= m_pWeightingValues.length)) return;

    if (iValue < 0) iValue = 0;
    if (iValue > 255) iValue = 255;

    m_pWeightingValues[iItemId] = iValue;

  }
  
  
  //////////////////////////
  // GET METHODS          //
  //////////////////////////
  public int getWeight(int iItemId) {
    //1. make some checks
    if (iItemId < 0) return 0;
    if (iItemId >= m_pWeightingValues.length) return 0;

    return m_pWeightingValues[iItemId];
  }

  public int[] getWeights() {
     return m_pWeightingValues;
  }

  
  ///////////////////////////////////////
  // LIST CELL RENDERER INTERFACE      //
  ///////////////////////////////////////
  // This is the only method defined by ListCellRenderer.
  // We just reconfigure the JLabel each time we're called.

  public Component getListCellRendererComponent(
    JList pList,
    Object pValue,            // value to display
    int iIndex,               // cell index
    boolean bIsSelected,      // is the cell selected
    boolean bCellHasFocus     // the list and the cell have the focus
  )    
  {
         
         //1. set the string value
         String pString = pValue.toString();
         setText(pString);

         
         //2. draw the selected item
         if (bIsSelected) {

           Color pColor;
           if ((iIndex >= 0) && (iIndex < m_pWeightingValues.length)) {
             double dNormVal = Math.exp(2) - 1;
             double dColWeight = (Math.exp(2*(double)m_pWeightingValues[iIndex]/255) - 1)/dNormVal;
             int iColValue = (int)((1-dColWeight)*255);

             pColor = new Color(0, iColValue, 255);
           }
           else {
             pColor = Color.BLUE;
           }

           setBackground(pColor);
           setForeground(pList.getSelectionForeground());
         }

         //3. draw unselected item
         else {
           Color pColor;

           if ((iIndex >= 0) && (iIndex < m_pWeightingValues.length)) {
              double dNormVal = Math.exp(2) - 1;
              double dColWeight = (Math.exp(2*(double)m_pWeightingValues[iIndex]/255) - 1)/dNormVal;
              int iColValue = (int)((1-dColWeight)*255);

              pColor = new Color(255, iColValue, iColValue);
           }
           else {
             pColor = Color.WHITE;
           }
           setBackground(pColor);
           setForeground(pList.getForeground());
         }
         
         
         setEnabled(pList.isEnabled());
         setFont(pList.getFont());
         setOpaque(true);
         return this;
     }
 }
