package view;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JInfoViz.
 *  
 *  JInfoViz is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JInfoViz is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JInfoViz.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.swing.*;
import java.awt.event.*;
import java.awt.Cursor;
import java.util.Vector;
import java.awt.geom.Point2D.Float;

//library imports
import data.ScatterplotSelection;
import data.SelectionList;
import listeners.GlobalConstants;
import listeners.IDataSubsetListener;



/* class handling the interactive creation and manipulation of selections for the scatterplot view */
public class OutlierDetectionInteraction extends Interaction implements GlobalConstants, ActionListener {


  /** member variables **/
  private JPopupMenu              m_pAxisSelPopupMenu;              // popup menu for the AxisSelections
  private OutlierDetectionView    m_pOutlierDetectionView;    // the outlier detection view






  /** constructor **/
  public OutlierDetectionInteraction(
    Vector<IDataSubsetListener> pListeners, 
    OutlierDetectionView pOutlierDetectionView
  ) {
    super(pListeners);

    //set member variables
    m_pOutlierDetectionView = pOutlierDetectionView;

    //create the popup menus
    createAxisSelectionPopupMenu();


  }



  ////////////////////////////////////////
  //MouseListener methods               //
  ////////////////////////////////////////
  //mouse clicked: show selection menu
  public void mouseClicked(MouseEvent e) {
      m_pAxisSelPopupMenu.show(m_pOutlierDetectionView, e.getX(), e.getY());
  }



  public void mouseEntered(MouseEvent e) {}
  public void mouseExited(MouseEvent e) {}


  //mouse button pressed
  public void mousePressed(MouseEvent e) {}



  //mouse is released:
  public void mouseReleased(MouseEvent e) {}


  ////////////////////////////////////////
  //MouseMotionListener methods         //
  ////////////////////////////////////////
  public void mouseDragged(MouseEvent e) {}

  public void mouseMoved(MouseEvent e) {}


  //////////////////////////////////////////
  // interface for the key events         //
  // triggered from the mainframe         //
  //////////////////////////////////////////
  public void setControlPressed(boolean pressed) {
  }





  ////////////////////////////////////////
  //POPUP MENUS                         //
  ////////////////////////////////////////
  private void createAxisSelectionPopupMenu() {

    m_pAxisSelPopupMenu = new JPopupMenu();

    JMenuItem pWorkWithOutliers = new JMenuItem("Work with outliers");
    pWorkWithOutliers.addActionListener(this);
    m_pAxisSelPopupMenu.add(pWorkWithOutliers);

    JMenuItem pRejectOutliers = new JMenuItem("Reject outliers");
    pRejectOutliers.addActionListener(this);
    m_pAxisSelPopupMenu.add(pRejectOutliers);

  }


  //////////////////////////////////////////
  //ActionListeners: for popup menu items //
  //////////////////////////////////////////
  public void actionPerformed(ActionEvent e) {

    if (e.getActionCommand().equals("Work with outliers")) {
      float[] pDOI = m_pOutlierDetectionView.getDOIofOutliers();
      setSelectedDataAsData(pDOI);
      return;
    }

    if (e.getActionCommand().equals("Reject outliers")) {
      float[] pDOI = m_pOutlierDetectionView.getDOIofNonOutliers();
      setSelectedDataAsData(pDOI);
      return;
    }

  }



}